//Illinois Open Source License
//
//University of Illinois
//Open Source License
//
//Copyright © 2014,    Board of Trustees of the University of Illinois.  All rights reserved.
//
//Developed by:
//
// Distributed Protocols Research Group in the Department of Computer Science
// The University of Illinois at Urbana-Champaign
// http://dprg.cs.uiuc.edu/
// This is for the Project 'Leveraging Metadata in NoSQL Storage Systems'. The paper can be found at the website http://dprg.cs.uiuc.edu
// The project can be run with Wasef enabled using the the script "start_wcass.sh"
//      People:
//	Ala' Alkhaldi, aalkhal2@illinois.edu
//	Indranil Gupta, indy@cs.uiuc.edu
//	Vaijayanth Raghavan, vraghvn2@illinois.edu
// 	Mainak Ghosh, mghosh4@illinois.edu
//Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal with the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimers.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimers in the documentation and/or other materials provided with the distribution.
//    * Neither the names of The Distributed Protocols Research Group (DPRG) or The University of Illinois at Urbana-Champaign, nor the names of its contributors may be used to endorse or promote products derived from this Software without specific prior written permission.
//
//THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
//AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.


package org.apache.cassandra.metadata;

import java.util.Arrays;

import org.apache.cassandra.concurrent.Stage;
import org.apache.cassandra.concurrent.StageManager;
import org.apache.cassandra.db.ConsistencyLevel;
import org.apache.cassandra.db.RowMutation;
import org.apache.cassandra.service.StorageProxy;
import org.apache.cassandra.utils.WrappedRunnable;

public class Metadata {

	public static final String MetaData_KS = "system_metadata";

	public static final String MetadataRegistry_CF = "registry";
	public static final String MetadataLog_CF = "log";

	public static final String AlterKeyspace_Tag = "a_ks";
	public static final String DropKeyspace_Tag = "d_ks";

	public static final String AlterColumnFamily_Alter_Tag = "a_cf_al";
	public static final String AlterColumnFamily_Add_Tag = "a_cf_ad";
	public static final String AlterColumnFamily_Drop_Tag = "a_cf_d";
	public static final String AlterColumnFamily_Rename_Tag = "a_cf_r";
	public static final String AlterColumnFamily_Prob_Tag = "a_cf_p";
	public static final String DropColumnFamily_Tag = "d_cf";
	public static final String TruncateColumnFamily_Tag = "t_cf";

	public static final String Insert_Tag = "i_r";
	public static final String Update_Tag = "u_r";
	public static final String delete_Tag = "d_r";

	public static void mutate(final RowMutation mutation){
		StageManager.getStage(Stage.MUTATION).execute(new WrappedRunnable() {
			public void runMayThrow() throws Exception {
				StorageProxy.mutate(Arrays.asList(mutation),ConsistencyLevel.ANY);
			}
		});
	}
}
