//Illinois Open Source License
//
//University of Illinois
//Open Source License
//
//Copyright © 2014,    Board of Trustees of the University of Illinois.  All rights reserved.
//
//Developed by:
//
// Distributed Protocols Research Group in the Department of Computer Science
// The University of Illinois at Urbana-Champaign
// http://dprg.cs.uiuc.edu/
// This is for the Project 'Leveraging Metadata in NoSQL Storage Systems'. The paper can be found at the website http://dprg.cs.uiuc.edu
// The project can be run with Wasef enabled using the the script "start_wcass.sh"
//      People:
//	Ala' Alkhaldi, aalkhal2@illinois.edu
//	Indranil Gupta, indy@cs.uiuc.edu
//	Vaijayanth Raghavan, vraghvn2@illinois.edu
// 	Mainak Ghosh, mghosh4@illinois.edu
//Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal with the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimers.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimers in the documentation and/or other materials provided with the distribution.
//    * Neither the names of The Distributed Protocols Research Group (DPRG) or The University of Illinois at Urbana-Champaign, nor the names of its contributors may be used to endorse or promote products derived from this Software without specific prior written permission.
//
//THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
//AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.


package org.apache.cassandra.metadata;

import java.util.ArrayList;

import org.apache.cassandra.db.ColumnFamilyStore;
import org.apache.cassandra.db.Table;

import java.lang.reflect.*;

public class MetricsCollector {
	
	private String[] target;
	private String[] metrics;
	
	public MetricsCollector(String target, String metrics){
		this.metrics = metrics.split(";");
		this.target = target.split("\\.");
	}

	public String getMetrics() {
		String collectedMetrics = "";
		try{
			for(String metric: metrics){
				
				String[] specs = metric.split(",");
				String type = specs[0].trim().substring(specs[0].trim().indexOf("type=")+5);
				String scope = specs[1].trim().substring(specs[0].trim().indexOf("scope=")+7);
				String name = specs[2].trim().substring(specs[0].trim().indexOf("name=")+6);
				
				if(type.equals("ColumnFamily") || metric.equals("IndexColumnFamily")){
					String keyspace = target[0];
					String columnFamily = "user"; //target[1];
					collectedMetrics += getColumnFamilyMetrics(keyspace, columnFamily, name);
				}
			}
		}catch(Exception ex){
			collectedMetrics = "";
		}
		return collectedMetrics;
	}

	
	
	private String getColumnFamilyMetrics(String keyspace, String columnFamily, String metricName){
		
		String collectedMetrics;
		try{
			ColumnFamilyStore cfs = Table.open(keyspace).getColumnFamilyStore(columnFamily);
			
			Method method = cfs.getClass().getMethod("get" + metricName, null);
			
			collectedMetrics =  metricName + ":" + method.invoke(cfs, null).toString();
		}catch(Exception ex){
			collectedMetrics = "";
		}
		return collectedMetrics;
	}


}
