//Illinois Open Source License
//
//University of Illinois
//Open Source License
//
//Copyright © 2014,    Board of Trustees of the University of Illinois.  All rights reserved.
//
//Developed by:
//
// Distributed Protocols Research Group in the Department of Computer Science
// The University of Illinois at Urbana-Champaign
// http://dprg.cs.uiuc.edu/
// This is for the Project 'Leveraging Metadata in NoSQL Storage Systems'. The paper can be found at the website http://dprg.cs.uiuc.edu
// The project can be run with Wasef enabled using the the script "start_wcass.sh"
//      People:
//	Ala' Alkhaldi, aalkhal2@illinois.edu
//	Indranil Gupta, indy@cs.uiuc.edu
//	Vaijayanth Raghavan, vraghvn2@illinois.edu
// 	Mainak Ghosh, mghosh4@illinois.edu
//Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal with the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimers.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimers in the documentation and/or other materials provided with the distribution.
//    * Neither the names of The Distributed Protocols Research Group (DPRG) or The University of Illinois at Urbana-Champaign, nor the names of its contributors may be used to endorse or promote products derived from this Software without specific prior written permission.
//
//THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
//AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.package org.apache.cassandra.config;

import org.apache.cassandra.OrderedJUnit4ClassRunner;
import org.apache.cassandra.SchemaLoader;
import org.apache.cassandra.exceptions.ConfigurationException;
import org.apache.cassandra.exceptions.InvalidRequestException;
import org.apache.cassandra.gms.Gossiper;
import org.apache.cassandra.locator.SimpleStrategy;
import org.apache.cassandra.service.MigrationManager;

import org.junit.Test;
import org.junit.runner.RunWith;

import java.io.IOException;

@RunWith(OrderedJUnit4ClassRunner.class)
public class DatabaseDescriptorTest
{
    @Test
    public void testCFMetaDataSerialization() throws IOException, ConfigurationException, InvalidRequestException
    {
        // test serialization of all defined test CFs.
        for (String table : Schema.instance.getNonSystemTables())
        {
            for (CFMetaData cfm : Schema.instance.getTableMetaData(table).values())
            {
                CFMetaData cfmDupe = CFMetaData.fromThrift(cfm.toThrift());
                assert cfmDupe != null;
                assert cfmDupe.equals(cfm);
            }
        }
    }

    @Test
    public void testKSMetaDataSerialization() throws IOException, ConfigurationException
    {
        for (KSMetaData ksm : Schema.instance.getTableDefinitions())
        {
            // Not testing round-trip on the KsDef via serDe() because maps
            //  cannot be compared in avro.
            KSMetaData ksmDupe = KSMetaData.fromThrift(ksm.toThrift());
            assert ksmDupe != null;
            assert ksmDupe.equals(ksm);
        }
    }

    // this came as a result of CASSANDRA-995
    @Test
    public void testTransKsMigration() throws IOException, ConfigurationException
    {
        SchemaLoader.cleanupAndLeaveDirs();
        DatabaseDescriptor.loadSchemas();
        assert Schema.instance.getNonSystemTables().size() == 0;

        Gossiper.instance.start((int)(System.currentTimeMillis() / 1000));

        try
        {
            // add a few.
            MigrationManager.announceNewKeyspace(KSMetaData.testMetadata("ks0", SimpleStrategy.class, KSMetaData.optsWithRF(3)));
            MigrationManager.announceNewKeyspace(KSMetaData.testMetadata("ks1", SimpleStrategy.class, KSMetaData.optsWithRF(3)));

            assert Schema.instance.getTableDefinition("ks0") != null;
            assert Schema.instance.getTableDefinition("ks1") != null;

            Schema.instance.clearTableDefinition(Schema.instance.getTableDefinition("ks0"));
            Schema.instance.clearTableDefinition(Schema.instance.getTableDefinition("ks1"));

            assert Schema.instance.getTableDefinition("ks0") == null;
            assert Schema.instance.getTableDefinition("ks1") == null;

            DatabaseDescriptor.loadSchemas();

            assert Schema.instance.getTableDefinition("ks0") != null;
            assert Schema.instance.getTableDefinition("ks1") != null;
        }
        finally
        {
            Gossiper.instance.stop();
        }
    }
}
